/*
  ------------------------------------------------------------------------------

    Copyright © 2005-2011, Tommi T. Järvi
                           tommi . jarvi @ iki . fi (remove spaces)


    This file is part of one of my programs.

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2,
    as published by the Free Software Foundation.

    This is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  ------------------------------------------------------------------------------
*/


#include "cnn.h"


/*
   *************************************
   *** CNA                           ***
   *************************************
*/


void CNA::add_unknown(const CNA_Signature &sig, int &newtype) {

  /*
    Add unknown signature to list if not already there.

    For unknown types not yet encountered, newtype is determined as the lowest
    unused integer type number. For already encountered ones, the previously determined
    number is given.
  */


  // --- variables
  int i;  // loops
  int n;  // new type for unknown signature
  bool used;
  map<CNA_Signature, int>::iterator j;  // loops, unknowns


  // --- count for different unknown signatures
  unknowns_count[sig]++;


  // --- give / find type

  // for new unknown types
  if(unknowns_count[sig]==1) {

    // new unknown type
    n = 0;
    used = true;
    while(used) {
      n++;
      used = false;

      // look in known types
      for(i=0;i<type.size();i++) {
	if(n==type[i]) {
	  // n in use
	  used = true;
	  break;
	}
      }

      // look in unknown types
      if(!used) {
	j = unknowns_type.begin();
	while(j != unknowns_type.end()) {
	  if(n==(*j).second) {
	    // n in use
	    used = true;
	    break;
	  }
	  j++;
	}
      }
    }  // new unknown type determined

    newtype = n;
    unknowns_type[sig] = newtype;
  }

  // for known unknown types
  else {
    newtype = unknowns_type[sig];
  }
}


void CNA::print_stats(double time) {

  /*
    Print statistics on bond signatures.
  */


  // --- variables
  map<CNA_Number, int>::iterator i;  // loops


  // --- bond signature statistics

  cout << "CNA::print_stats(): Outputting CNA bond signature statistics: (time, signature, count)" << endl;
  cout << "------------------------------------------------------------" << endl;


  i = stats.begin();
  while(i != stats.end()) {
    cout << "cnabond ";
    cout.width(10);
    cout << time << " ";
    cout.width(2);
    cout << (*i).first.nn << " ";
    cout.width(2);
    cout << (*i).first.n << " ";
    cout.width(2);
    cout << (*i).first.b << " ";
    cout.width(2);
    cout << (*i).first.t << " ";
    cout.width(10);
    cout << (*i).second/2.0 << endl;
    i++;
  }

  cout << "------------------------------------------------------------" << endl;
}


void CNA::print_unknowns() {

  /*
    Print unknown signatures in a form suitable for
    copy-pasting back to the source code.
  */


  // --- variables
  map<CNA_Signature, int>::iterator i;  // loops
  map<CNA_Number, int>::iterator j;  // loops
  CNA_Signature temps;  // workaround for not being able to iterate over CNA_Signature::bonds
  int k;  // loops
  int chunk;  // check for unknown signatures
  int n;  // for output


  // --- print

  cout << "-------------------------------" << endl;
  i = unknowns_count.begin();
  chunk = 0;
  n = 0;
  while(i != unknowns_count.end()) {
    n++;
    cout << "Unknown CNA-signature " << n << " (count " << (*i).second << ")" << endl;
    cout << "-------------------------------" << endl;
    cout << "   sig.reset();" << endl;
    temps = (*i).first;
    j = temps.bonds.begin();
    while(j != temps.bonds.end()) {
      for(k=0;k<(*j).second;k++) {
	cout << "   sig.add(CNA_Number(" << (*j).first.nn << ", " << (*j).first.n << ", " << (*j).first.b << ", " << (*j).first.t << "));" << endl;
      }
      j++;
    }
    cout << "   data.push_back(sig);" << endl;
    cout << "   name.push_back(NAME);" << endl;
    cout << "   type.push_back(TYPE);" << endl;
    cout << "-------------------------------" << endl;

    // check count
    chunk += (*i).second;

    i++;
  }


  // --- report count check
  cout << "CNA::print_unknowns(): Found " << chunk << " atoms with unknown signature. That should match with the value above." << endl;
}


void CNA::savesigs(string file) {

  /*
    Save all signatures (read in + new unknown ones) to a file that can
    be read in by CNA::readsigs().
  */


  // --- variables
  int k, l;  // loops
  ofstream f;  // output file
  map<CNA_Signature, int>::iterator i;  // loops, unknowns
  map<CNA_Number, int>::iterator j;  // loops, unknowns and knowns
  CNA_Signature temps;  // workaround for not being able to iterate over CNA_Signature::bonds
  int n;  // for output


  // --- open file
  cout << "CNA::savesigs(): Saving CNA signatures to file " << file << endl;
  f.open(file.c_str(), ios::trunc);


  // --- write file

  // total number of signatures
  f << data.size() + unknowns_count.size() << endl;

  // output known signatures
  for(l=0;l<data.size();l++) {

    // type number and number of bonds
    f << type[l] << " " << data[l].nbonds << endl;

    // comment / name
    f << name[l] << endl;

    // bonds
    j = data[l].bonds.begin();
    while(j != data[l].bonds.end()) {
      for(k=0;k<(*j).second;k++) {
	f << (*j).first.nn << " " << (*j).first.n << " " << (*j).first.b << " " << (*j).first.t << endl;
      }
      j++;
    }
  }  // end of output known signatures

  // output unknown signatures
  i = unknowns_count.begin();
  while(i != unknowns_count.end()) {
    temps = (*i).first;

    // type number and number of bonds
    f << unknowns_type[temps] << " " << temps.nbonds << endl;

    // comment / name
    f << "unknown " << unknowns_type[temps] << " count " << (*i).second << endl;

    // bonds
    j = temps.bonds.begin();
    while(j != temps.bonds.end()) {
      for(k=0;k<(*j).second;k++) {
	f << (*j).first.nn << " " << (*j).first.n << " " << (*j).first.b << " " << (*j).first.t << endl;
      }
      j++;
    }
    i++;
  }

}


void CNA::calculate_and_analyze(Atom_System &s, double time) {

  /*
    Calculates the common neighbour analysis signature for
    a system of atoms.

    Probably not the most efficient way to do this.

    Modes

    1         Only pairs with a bond between them
    2         Same but surface atoms marked with 5
  */


  // --- variables

  int i, j, k, m, n, l, o;  // loops, temps
  int maxpairs = 100;  // max number of neareast neighbours
  int maxcommnei = 10;  // max number of common neighbours
  int ncommnei;  // number of common neighbours of current atom
  int nbondnei;  // number of bonds between these
  bool take;  // this neighbour already there?
  CNA_Number number;  // characteristic number of the current pair
  valarray<int> commnei;  // common neighbours
  int natoms;
  CNA_Signature thissig;  // CNA signature of current atom

  // analyze
  bool match;  // structure identified
  valarray<int> count;  // statistics -''-
  int count_unknown;  // statistics for unknown atoms
  int count_surface;  // statistics for surface atoms (mode 2)
  int count_tot;  // total number of identified atoms
  int newtype;  // type number given to unknown signature


  // --- check
  if(bondcut < 0) {
    cout << "CNA::calculate(): Specify bond cutoff." << endl;
    exit(-1);
  }

  if(mode != 0 && mode != 1 && mode != 2) {
    cout << "CNA::calculate(): Unknown mode: " << mode << endl;
    exit(-1);
  }


  // --- init analyse

  if(!type.empty()) {
    count.resize(type.size());
  }
  for(i=0;i<count.size();i++) {
    count[i] = 0;
  }
  count_unknown = 0;
  count_surface = 0;


  // --- init array

  natoms = s.atoms();
  cout << "CNA::calculate(): Init arrays. Bondcut " << bondcut << ", atoms: " << natoms << endl;

  commnei.resize(maxcommnei);


  // --- clear unknown signatures
  unknowns_count.clear();
  unknowns_type.clear();


  // --- calculate

  cout << "CNA::calculate(): Calculating signatures...";

  // loop over atoms
  for(i=0;i<natoms;i++) {
    thissig.reset();

    // loop over other atoms
    for(o=1;o<=s.symmnlist[i][0];o++) {
      j = s.symmnlist[i][o];

      // don't take atoms with themselves
      if(i==j) {
	cerr << "CNA::calculate(): Error! Bug, report it." << endl;
	exit(-1);
      }

      // skip for pairs with no possible common neighbours
      if(s.dr(i,j) > 2.01*bondcut) continue;

      // if in mode 1, 2, take only pairs with a bond between the atoms
      if((mode==1 || mode==2) && s.dr(i,j) > bondcut) continue;

      // --- find common neighbours

      // resize common neighbour array if needed
      if(s.symmnlist[i][0] < s.symmnlist[j][0]) k = s.symmnlist[j][0];
      else k = s.symmnlist[i][0];
      if(maxcommnei < k) {
	maxcommnei = k;
	commnei.resize(maxcommnei);
      }

      // loop over neighbours of atom i
      ncommnei = 0;
      for(k=1;k<=s.symmnlist[i][0];k++) {
	if(s.dr(s.symmnlist[i][k],j) > bondcut) {
          continue;
        }

	// loop over neighbours of atom j
	for(m=1;m<=s.symmnlist[j][0];m++) {
	  if(s.dr(s.symmnlist[j][m],i) > bondcut) {
            continue;
          }

	  // take neighbour
	  if(s.symmnlist[i][k]==s.symmnlist[j][m]) {
	    take = true;

            // don't take i or j as a common neighbour of i and j
            if(s.symmnlist[i][k]==i || s.symmnlist[i][k]==j) {
              cerr << "CNA::calculate(): Error: Atom is its own neighbour." << endl;
              exit(-1);
            }

            // check if this neighbour is listed already
	    for(n=0;n<ncommnei;n++) {
	      if(commnei[n]==s.symmnlist[i][k]) take = false;
	    }

            // take
	    if(take) {
	      commnei[ncommnei] = s.symmnlist[i][k];
	      ncommnei++;
	    }
	  }  // neighbour taken

	}  // end of loop over neighbours of atom j
      }  // end of loop over neighbours of atom i


      // --- skip for atoms with no common neighbours
      //if(ncommnei==0) continue;


      // --- common neighbours found, count bonds between them
      nbondnei = 0;
      for(k=0;k<ncommnei;k++) {
	for(m=0;m<ncommnei;m++) {

	  if(k<=m) continue;

	  if(s.dr(commnei[k], commnei[m]) < bondcut) {
	    nbondnei++;
	  }
	}
      }


      // --- all numbers calculated, save
      number.n = ncommnei;
      number.b = nbondnei;
      if(s.dr(i, j) < bondcut) number.nn = 1;
      else number.nn = 0;

      // topological factor, for fcc
      if(ncommnei==4 && nbondnei==2) {

	// find largest number of bonds to the common neighbours by one atom
	l = 0;
	for(k=0;k<ncommnei;k++) {
	  n = 0;
	  for(m=0;m<ncommnei;m++) {

	    if(k==m) continue;

	    if(s.dr(commnei[k], commnei[m]) < bondcut) {
	      n++;
	    }
	  }

	  if(n>l) l = n;
	}

	// set number
	if(l==2) {
	  number.t = 2;
	}
	else {
	  number.t = 1;
	}
      }
      else number.t = 1;


      // --- set numbers
      thissig.add(number);


      // --- bond signature statistics
      stats[number]++;


    }  // end of loop over other atoms


  /*
    Identify atoms with known structure.
  */

    match = false;

    // compare to known signatures
    for(j=0;j<data.size();j++) {
      if(thissig == data[j]) {
	match = true;
	s.V[i] = type[j];
	count[j]++;
	break;
      }
    }

    // surface atoms for mode 2
    if(mode == 2 && !match) {
      if(thissig.nbonds < 10) {
	match = true;
	s.V[i] = 5;
	count_surface++;
      }
    }

    // collect unidentified signatures
    if(!match && track_unknowns) {
      add_unknown(thissig, newtype);
      s.V[i] = newtype;
      count_unknown++;
    }
    if(!match && !track_unknowns) {
      //    else {
      s.V[i] = 0;
      count_unknown++;
    }
  }  // end of loop over atoms


  // --- report
  cout << " ...done." << endl;


  // --- identified atom statistics

  cout << "CNA::analyse(): Outputting CNA identified atom type statistics: (time, signature, count, explanation)" << endl;
  cout << "------------------------------------------------------------" << endl;

  count_tot = 0;

  cout << "cnaatom ";
  cout.width(10);
  cout << time << " ";
  cout.width(4);
  cout << 0 << " ";
  cout.width(10);
  cout << count_unknown << " unknown" << endl;
  count_tot += count_unknown;

  for(i=0;i<type.size();i++) {
    cout << "cnaatom ";
    cout.width(10);
    cout << time << " ";
    cout.width(4);
    cout << type[i] << " ";
    cout.width(10);
    cout << count[i] << " ";
    count_tot += count[i];
    cout << name[i] << endl;
  }

  // mode 2, surface atoms
  if(mode==2) {
    cout << "cnaatom ";
    cout.width(10);
    cout << time << " ";
    cout.width(4);
    cout << 5 << " ";
    cout.width(10);
    cout << count_surface << " surface" << endl;
    count_tot += count_surface;
  }

  cout << "------------------------------------------------------------" << endl;


  // check
  if(count_tot != s.atoms()) {
    cerr << "CNA::analyse(): Error: There are " << s.atoms() << " atoms in the system but I identified " << count_tot << "." << endl;
    exit(-1);
  }
}


void CNA::readsigs(string file) {

  /*
    Read known CNA signatures from a file.

    Format:
    ----------------
    2                   number of signatures
    1 2                 id for 1st sign. + number of bonds
    comment line
    1 2 3 4             1st bond of 1st sign. (nn n b t)
    3 2 4 5
    2 3
    .
    .
    .
    ----------------

    Above, the bond numbers are
    nn  1 if atoms are neighbours, 0 otherwise
    n   number of common neighbours
    b   bonds between common neighbours
    t   topology
  */


  // --- variables
  int i, j;  // loops
  ifstream afile;  // input file
  int nsig;  // number of signatures
  int nbonds;  // number of bonds in current signature
  int id;  // type number of current signature
  string comment;  // comment of current signature
  CNA_Signature sig;  // current signature
  // bond numbers
  int nn;  // 1 if atoms are neighbours, 0 otherwise
  int n;  // number of common neighbours
  int b;  // bonds between common neighbours
  int t;  // topology


  // --- prepare

  // clear possible previous data
  data.clear();
  name.clear();
  type.clear();

  // open file
  cout << "CNA::readsigs(): Reading known CNA signatures from file " << file << endl;
  afile.open(file.c_str());
  assert(afile);


  // --- read file

  // number of signatures
  afile >> nsig;

  // loop over signatures
  for(i=0;i<nsig;i++) {
    // reset
    sig.reset();

    // id
    afile >> id;

    // check that id is unique
    for(j=0;j<type.size();j++) {
      if(id==type[j]) {
	cerr << "CNA::readsigs(): Trying to add two signatures with same id number (" << id << ")." << endl;
	exit(-1);
      }
    }
    type.push_back(id);

    // number of bonds
    afile >> nbonds;

    // comment
    getline(afile, comment, '\n');  // end of previous line
    getline(afile, comment, '\n');
    name.push_back(comment);

    // read bonds
    for(j=0;j<nbonds;j++) {
      afile >> nn;
      afile >> n;
      afile >> b;
      afile >> t;
      sig.add(CNA_Number(nn, n, b, t));
    }
    data.push_back(sig);

  }  // end of loop over signatures
}


/*
   *************************************
   *** CNA_Number                    ***
   *************************************
*/


CNA_Number& CNA_Number::operator=(const CNA_Number &other) {

  /*
    Operator =
  */

  // don't copy self
  if(this == &other) return *this;

  nn = other.nn;
  n = other.n;
  b = other.b;
  t = other.t;

  return *this;
}


bool CNA_Number::operator<(const CNA_Number &other) const {

  /*
    c1 < c2   true
    c2 < c1   false
    c1 = c2   false

    Has to be defined to use Map.
  */


  if(nn < other.nn) return true;
  else if(nn > other.nn) return false;
  else if(n < other.n) return true;
  else if(n > other.n) return false;
  else if(b < other.b) return true;
  else if(b > other.b) return false;
  else if(t < other.t) return true;
  else if(t > other.t) return false;
  else return false;
}
