/*
  ------------------------------------------------------------------------------

    Copyright © 2005-2011, Tommi T. Järvi
                           tommi . jarvi @ iki . fi (remove spaces)


    This file is part of one of my programs.

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2,
    as published by the Free Software Foundation.

    This is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  ------------------------------------------------------------------------------
*/


#include "mathsubs.h"


double gaussian() {

  /*
    Generate Gaussian random numbers using
    gaussian(sigma) with sigma = 0.
  */

  return gaussian(0);
}


double gaussian(double sigma) {

  /*
    Generates random numbers with a Gaussian distribution
    with standard deviation "sigma". Adapted from W. H.
    Press et al., Numerical Recipes, 1st ed.

    Uses double genrand() to generate uniform random
    numbers in ]0,1[.
  */


  // --- variables
  double v1, v2, w, fac;  // in unit circle
  static double y;
  static int yset = 0;
  static double prevsigma = 0;


  // --- generate
  if(yset == 0 || sigma != prevsigma) {
    do {
      v1 = 2*genrand() - 1;
      v2 = 2*genrand() - 1;
      w = v1*v1 + v2*v2;
    }
    while (w >= 1.0 || w == 0.0);

    fac = sigma*sqrt(-2*log(1-w)/w);
    y = fac*v1;
    yset = 1;
    prevsigma = sigma;

    return fac*v2;
  }
  else {
    yset = 0;
    return y;
  }
}


string stringify(int n) {

  /*
    Convert an integer to a string.
  */

  ostringstream o(ostringstream::out);

  if (!(o << n)) {
    cerr << "Cannot convert " << n << " to string." << endl;
    exit(-1);
  }

  return o.str();
}


string stringify(double n) {

  /*
    Convert a double to a string.
  */

  ostringstream o(ostringstream::out);

  if (!(o << n)) {
    cerr << "Cannot convert " << n << " to string." << endl;
    exit(-1);
  }

  return o.str();
}


double norm(valarray<double> &x) {

  /*
    Norm of a vector.
  */

  int i;
  double abs = 0;

  for(i=0;i<x.size();i++) {
    abs += x[i]*x[i];
  }

  return sqrt(abs);
}


double normsq(valarray<double> &x) {

  /*
    Squared norm of a vector.
  */

  int i;
  double abs = 0;

  for(i=0;i<x.size();i++) {
    abs += x[i]*x[i];
  }

  return abs;
}


void normalto3v(valarray<double> &x, const valarray<double> &normal) {

  /*
    Takes the component of x normal to the vector "normal". "normal"
    must be normalised. x is normalised for each 3 components, as in
    the coordinate vector x0.
  */

  int i, j;
  double dotp;
  int natoms;

  // check
  if(normal.size() != x.size()) {
    cerr << "normalto3v(): normal doesn't match with x." << endl;
    exit(-1);
  }

  natoms = x.size();
  if(natoms % 3 != 0) {
    cerr << "normalto3v(): Error: Input vector length % 3 != 0." << endl;
    exit(-1);
  }
  natoms /= 3;

  for(i=0;i<natoms;i++) {

    dotp = 0;
    for(j=0;j<3;j++) {
      dotp += normal[3*i + j]*x[3*i + j];
    }

    for(j=0;j<3;j++) {
      x[3*i + j] -= dotp*normal[3*i + j];
    }
  }
}


void normalise3v(valarray<double> &x) {

  /*
    Normalises each 3-vector in x to 1.
  */

  int i, j;
  int natoms;
  double abs;

  natoms = x.size();
  if(natoms % 3 != 0) {
    cerr << "normalise3v(): Error: Input vector length % 3 != 0." << endl;
    exit(-1);
  }
  natoms /= 3;

  // loop over atoms
  for(i=0;i<natoms;i++) {

    // loop over dimensions
    abs = 0;
    for(j=0;j<3;j++) {
      abs += x[3*i + j]*x[3*i + j];
    }
    abs = sqrt(abs);
    if(abs == 0) abs = 1;

    // normalise
    for(j=0;j<3;j++) {
      x[3*i + j] /= abs;
    }
  }
}


void normalto3v(double *x, int n, const valarray<double> &normal) {

  /*
    Takes the component of x normal to the vector "normal". "normal"
    must be normalised. x is normalised for each 3 components, as in
    the coordinate vector x0.
  */

  int i, j;
  double dotp;
  int natoms;

  // check
  if(normal.size() != n) {
    cerr << "normalto3v(): normal doesn't match with x." << endl;
    exit(-1);
  }

  natoms = n;
  if(natoms % 3 != 0) {
    cerr << "normalto3v(): Error: Input vector length % 3 != 0." << endl;
    exit(-1);
  }
  natoms /= 3;

  for(i=0;i<natoms;i++) {

    dotp = 0;
    for(j=0;j<3;j++) {
      dotp += normal[3*i + j]*x[3*i + j];
    }

    for(j=0;j<3;j++) {
      x[3*i + j] -= dotp*normal[3*i + j];
    }
  }
}


double* grow(double *x, int n1, int n2) {

  /*
    Resize array from n1 to n2 while retaining elements.
  */

  int i;
  int a;
  double* temp;

#if defined(DEBUG)
  cout << "grow(): Resizing double-array: " << n1 << " -> " << n2 << endl;
#endif

  if(n1==n2) return x;

  if(n2==0) {
    if(n1 == 1) delete x;
    else if(n1 > 1) delete [] x;
    return NULL;
  }
  else if(n2 < 0) {
    cerr << "grow(): n1 n2 " << n1 << " " << n2 << endl;
    exit(-1);
  }

  temp = new double[n2];

  if(n1 < n2) a = n1;
  else a = n2;

  for(i=0;i<a;i++) {
    temp[i] = x[i];
  }

  if(n1 == 1) delete x;
  else if(n1 > 1) delete [] x;


  return temp;
}


int* grow(int *x, int n1, int n2) {

  /*
    Resize array from n1 to n2.
  */

  int i;
  int a;
  int* temp;

#if defined(DEBUG)
  cout << "grow(): Resizing int-array: " << n1 << " -> " << n2 << endl;
#endif

  if(n1==n2) return x;

  if(n2==0) {
    if(n1 == 1) delete x;
    else if(n1 > 1) delete [] x;
    return NULL;
  }
  else if(n2 < 0) {
    cerr << "grow(): n1 n2 " << n1 << " " << n2 << endl;
    exit(-1);
  }

  temp = new int[n2];

  if(n1 < n2) a = n1;
  else a = n2;

  for(i=0;i<a;i++) {
    temp[i] = x[i];
  }

  if(n1 == 1) delete x;
  else if(n1 > 1) delete [] x;

  return temp;
}


int pickrandom(valarray<double> x) {

  /*
    Pick i with probability corresponding to x[i]. Normalise x[i] automatically.
  */


  // --- variables
  int i, j;  // loops
  double total;  // total probability, temp
  valarray<double> cumul;  // cumulative probability


  // --- check
  if(x.size() <= 1) {
    cerr << "pickrandom(): Error: x.size() " << x.size() << endl;
    exit(-1);
  }


  // --- normalise x
  total = 0;
  for(i=0;i<x.size();i++) {
    if(x[i] < 0) {
      cerr << "pickrandom(): Error: x[" << i << "] = " << x[i] << endl;
      exit(-1);
    }

    total += x[i];
  }

  for(i=0;i<x.size();i++) {
    x[i] /= total;
  }

  cumul.resize(x.size());
  for(i=0;i<x.size();i++) {
    cumul[i] = 0;
  }
  for(i=0;i<x.size();i++) {
    for(j=0;j<=i;j++) {
      cumul[i] += x[j];
    }
  }


  // --- check
  /*
  for(i=0;i<cumul.size();i++) {
    cout << "pickrandom(): Check: i cumul x " << i << " " << cumul[i] << " " << x[i] << endl;
  }
  */


  // --- pick index
  total = genrand();

  for(i=0;i<cumul.size();i++) {
    if(cumul[i] > total) break;
  }

  return i;
}


// --- STRINGS


string tolower(string st) {

  /*
    Converts a string to lowercase, not tested.
  */


  // --- variables
  int i;  // loops


  // --- convert
  for(i=0;i<st.length();i++) {
    st[i] = tolower(st[i]);
    //st[i] = toupper(st[i]);
  }

  return st;
}
